# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import dataclasses
import json  # type: ignore
import re
from typing import Any, Callable, Dict, List, Optional, Sequence, Tuple, Union
import warnings

from google.api_core import gapic_v1, path_template, rest_helpers, rest_streaming
from google.api_core import exceptions as core_exceptions
from google.api_core import retry as retries
from google.auth import credentials as ga_credentials  # type: ignore
from google.auth.transport.grpc import SslCredentials  # type: ignore
from google.auth.transport.requests import AuthorizedSession  # type: ignore
from google.protobuf import json_format
import grpc  # type: ignore
from requests import __version__ as requests_version

try:
    OptionalRetry = Union[retries.Retry, gapic_v1.method._MethodDefault]
except AttributeError:  # pragma: NO COVER
    OptionalRetry = Union[retries.Retry, object]  # type: ignore


from google.cloud.webrisk_v1beta1.types import webrisk

from .base import DEFAULT_CLIENT_INFO as BASE_DEFAULT_CLIENT_INFO
from .base import WebRiskServiceV1Beta1Transport

DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo(
    gapic_version=BASE_DEFAULT_CLIENT_INFO.gapic_version,
    grpc_version=None,
    rest_version=requests_version,
)


class WebRiskServiceV1Beta1RestInterceptor:
    """Interceptor for WebRiskServiceV1Beta1.

    Interceptors are used to manipulate requests, request metadata, and responses
    in arbitrary ways.
    Example use cases include:
    * Logging
    * Verifying requests according to service or custom semantics
    * Stripping extraneous information from responses

    These use cases and more can be enabled by injecting an
    instance of a custom subclass when constructing the WebRiskServiceV1Beta1RestTransport.

    .. code-block:: python
        class MyCustomWebRiskServiceV1Beta1Interceptor(WebRiskServiceV1Beta1RestInterceptor):
            def pre_compute_threat_list_diff(self, request, metadata):
                logging.log(f"Received request: {request}")
                return request, metadata

            def post_compute_threat_list_diff(self, response):
                logging.log(f"Received response: {response}")
                return response

            def pre_search_hashes(self, request, metadata):
                logging.log(f"Received request: {request}")
                return request, metadata

            def post_search_hashes(self, response):
                logging.log(f"Received response: {response}")
                return response

            def pre_search_uris(self, request, metadata):
                logging.log(f"Received request: {request}")
                return request, metadata

            def post_search_uris(self, response):
                logging.log(f"Received response: {response}")
                return response

        transport = WebRiskServiceV1Beta1RestTransport(interceptor=MyCustomWebRiskServiceV1Beta1Interceptor())
        client = WebRiskServiceV1Beta1Client(transport=transport)


    """

    def pre_compute_threat_list_diff(
        self,
        request: webrisk.ComputeThreatListDiffRequest,
        metadata: Sequence[Tuple[str, str]],
    ) -> Tuple[webrisk.ComputeThreatListDiffRequest, Sequence[Tuple[str, str]]]:
        """Pre-rpc interceptor for compute_threat_list_diff

        Override in a subclass to manipulate the request or metadata
        before they are sent to the WebRiskServiceV1Beta1 server.
        """
        return request, metadata

    def post_compute_threat_list_diff(
        self, response: webrisk.ComputeThreatListDiffResponse
    ) -> webrisk.ComputeThreatListDiffResponse:
        """Post-rpc interceptor for compute_threat_list_diff

        Override in a subclass to manipulate the response
        after it is returned by the WebRiskServiceV1Beta1 server but before
        it is returned to user code.
        """
        return response

    def pre_search_hashes(
        self, request: webrisk.SearchHashesRequest, metadata: Sequence[Tuple[str, str]]
    ) -> Tuple[webrisk.SearchHashesRequest, Sequence[Tuple[str, str]]]:
        """Pre-rpc interceptor for search_hashes

        Override in a subclass to manipulate the request or metadata
        before they are sent to the WebRiskServiceV1Beta1 server.
        """
        return request, metadata

    def post_search_hashes(
        self, response: webrisk.SearchHashesResponse
    ) -> webrisk.SearchHashesResponse:
        """Post-rpc interceptor for search_hashes

        Override in a subclass to manipulate the response
        after it is returned by the WebRiskServiceV1Beta1 server but before
        it is returned to user code.
        """
        return response

    def pre_search_uris(
        self, request: webrisk.SearchUrisRequest, metadata: Sequence[Tuple[str, str]]
    ) -> Tuple[webrisk.SearchUrisRequest, Sequence[Tuple[str, str]]]:
        """Pre-rpc interceptor for search_uris

        Override in a subclass to manipulate the request or metadata
        before they are sent to the WebRiskServiceV1Beta1 server.
        """
        return request, metadata

    def post_search_uris(
        self, response: webrisk.SearchUrisResponse
    ) -> webrisk.SearchUrisResponse:
        """Post-rpc interceptor for search_uris

        Override in a subclass to manipulate the response
        after it is returned by the WebRiskServiceV1Beta1 server but before
        it is returned to user code.
        """
        return response


@dataclasses.dataclass
class WebRiskServiceV1Beta1RestStub:
    _session: AuthorizedSession
    _host: str
    _interceptor: WebRiskServiceV1Beta1RestInterceptor


class WebRiskServiceV1Beta1RestTransport(WebRiskServiceV1Beta1Transport):
    """REST backend transport for WebRiskServiceV1Beta1.

    Web Risk v1beta1 API defines an interface to detect malicious
    URLs on your website and in client applications.

    This class defines the same methods as the primary client, so the
    primary client can load the underlying transport implementation
    and call it.

    It sends JSON representations of protocol buffers over HTTP/1.1

    """

    def __init__(
        self,
        *,
        host: str = "webrisk.googleapis.com",
        credentials: Optional[ga_credentials.Credentials] = None,
        credentials_file: Optional[str] = None,
        scopes: Optional[Sequence[str]] = None,
        client_cert_source_for_mtls: Optional[Callable[[], Tuple[bytes, bytes]]] = None,
        quota_project_id: Optional[str] = None,
        client_info: gapic_v1.client_info.ClientInfo = DEFAULT_CLIENT_INFO,
        always_use_jwt_access: Optional[bool] = False,
        url_scheme: str = "https",
        interceptor: Optional[WebRiskServiceV1Beta1RestInterceptor] = None,
        api_audience: Optional[str] = None,
    ) -> None:
        """Instantiate the transport.

        Args:
            host (Optional[str]):
                 The hostname to connect to.
            credentials (Optional[google.auth.credentials.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify the application to the service; if none
                are specified, the client will attempt to ascertain the
                credentials from the environment.

            credentials_file (Optional[str]): A file with credentials that can
                be loaded with :func:`google.auth.load_credentials_from_file`.
                This argument is ignored if ``channel`` is provided.
            scopes (Optional(Sequence[str])): A list of scopes. This argument is
                ignored if ``channel`` is provided.
            client_cert_source_for_mtls (Callable[[], Tuple[bytes, bytes]]): Client
                certificate to configure mutual TLS HTTP channel. It is ignored
                if ``channel`` is provided.
            quota_project_id (Optional[str]): An optional project to use for billing
                and quota.
            client_info (google.api_core.gapic_v1.client_info.ClientInfo):
                The client info used to send a user-agent string along with
                API requests. If ``None``, then default info will be used.
                Generally, you only need to set this if you are developing
                your own client library.
            always_use_jwt_access (Optional[bool]): Whether self signed JWT should
                be used for service account credentials.
            url_scheme: the protocol scheme for the API endpoint.  Normally
                "https", but for testing or local servers,
                "http" can be specified.
        """
        # Run the base constructor
        # TODO(yon-mg): resolve other ctor params i.e. scopes, quota, etc.
        # TODO: When custom host (api_endpoint) is set, `scopes` must *also* be set on the
        # credentials object
        maybe_url_match = re.match("^(?P<scheme>http(?:s)?://)?(?P<host>.*)$", host)
        if maybe_url_match is None:
            raise ValueError(
                f"Unexpected hostname structure: {host}"
            )  # pragma: NO COVER

        url_match_items = maybe_url_match.groupdict()

        host = f"{url_scheme}://{host}" if not url_match_items["scheme"] else host

        super().__init__(
            host=host,
            credentials=credentials,
            client_info=client_info,
            always_use_jwt_access=always_use_jwt_access,
            api_audience=api_audience,
        )
        self._session = AuthorizedSession(
            self._credentials, default_host=self.DEFAULT_HOST
        )
        if client_cert_source_for_mtls:
            self._session.configure_mtls_channel(client_cert_source_for_mtls)
        self._interceptor = interceptor or WebRiskServiceV1Beta1RestInterceptor()
        self._prep_wrapped_messages(client_info)

    class _ComputeThreatListDiff(WebRiskServiceV1Beta1RestStub):
        def __hash__(self):
            return hash("ComputeThreatListDiff")

        __REQUIRED_FIELDS_DEFAULT_VALUES: Dict[str, Any] = {
            "threatType": {},
            "constraints": {},
        }

        @classmethod
        def _get_unset_required_fields(cls, message_dict):
            return {
                k: v
                for k, v in cls.__REQUIRED_FIELDS_DEFAULT_VALUES.items()
                if k not in message_dict
            }

        def __call__(
            self,
            request: webrisk.ComputeThreatListDiffRequest,
            *,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Optional[float] = None,
            metadata: Sequence[Tuple[str, str]] = (),
        ) -> webrisk.ComputeThreatListDiffResponse:
            r"""Call the compute threat list diff method over HTTP.

            Args:
                request (~.webrisk.ComputeThreatListDiffRequest):
                    The request object. Describes an API diff request.
                retry (google.api_core.retry.Retry): Designation of what errors, if any,
                    should be retried.
                timeout (float): The timeout for this request.
                metadata (Sequence[Tuple[str, str]]): Strings which should be
                    sent along with the request as metadata.

            Returns:
                ~.webrisk.ComputeThreatListDiffResponse:

            """

            http_options: List[Dict[str, str]] = [
                {
                    "method": "get",
                    "uri": "/v1beta1/threatLists:computeDiff",
                },
            ]
            request, metadata = self._interceptor.pre_compute_threat_list_diff(
                request, metadata
            )
            pb_request = webrisk.ComputeThreatListDiffRequest.pb(request)
            transcoded_request = path_template.transcode(http_options, pb_request)

            uri = transcoded_request["uri"]
            method = transcoded_request["method"]

            # Jsonify the query params
            query_params = json.loads(
                json_format.MessageToJson(
                    transcoded_request["query_params"],
                    including_default_value_fields=False,
                    use_integers_for_enums=True,
                )
            )
            query_params.update(self._get_unset_required_fields(query_params))

            query_params["$alt"] = "json;enum-encoding=int"

            # Send the request
            headers = dict(metadata)
            headers["Content-Type"] = "application/json"
            response = getattr(self._session, method)(
                "{host}{uri}".format(host=self._host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params, strict=True),
            )

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

            # Return the response
            resp = webrisk.ComputeThreatListDiffResponse()
            pb_resp = webrisk.ComputeThreatListDiffResponse.pb(resp)

            json_format.Parse(response.content, pb_resp, ignore_unknown_fields=True)
            resp = self._interceptor.post_compute_threat_list_diff(resp)
            return resp

    class _SearchHashes(WebRiskServiceV1Beta1RestStub):
        def __hash__(self):
            return hash("SearchHashes")

        __REQUIRED_FIELDS_DEFAULT_VALUES: Dict[str, Any] = {
            "threatTypes": {},
        }

        @classmethod
        def _get_unset_required_fields(cls, message_dict):
            return {
                k: v
                for k, v in cls.__REQUIRED_FIELDS_DEFAULT_VALUES.items()
                if k not in message_dict
            }

        def __call__(
            self,
            request: webrisk.SearchHashesRequest,
            *,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Optional[float] = None,
            metadata: Sequence[Tuple[str, str]] = (),
        ) -> webrisk.SearchHashesResponse:
            r"""Call the search hashes method over HTTP.

            Args:
                request (~.webrisk.SearchHashesRequest):
                    The request object. Request to return full hashes matched
                by the provided hash prefixes.
                retry (google.api_core.retry.Retry): Designation of what errors, if any,
                    should be retried.
                timeout (float): The timeout for this request.
                metadata (Sequence[Tuple[str, str]]): Strings which should be
                    sent along with the request as metadata.

            Returns:
                ~.webrisk.SearchHashesResponse:

            """

            http_options: List[Dict[str, str]] = [
                {
                    "method": "get",
                    "uri": "/v1beta1/hashes:search",
                },
            ]
            request, metadata = self._interceptor.pre_search_hashes(request, metadata)
            pb_request = webrisk.SearchHashesRequest.pb(request)
            transcoded_request = path_template.transcode(http_options, pb_request)

            uri = transcoded_request["uri"]
            method = transcoded_request["method"]

            # Jsonify the query params
            query_params = json.loads(
                json_format.MessageToJson(
                    transcoded_request["query_params"],
                    including_default_value_fields=False,
                    use_integers_for_enums=True,
                )
            )
            query_params.update(self._get_unset_required_fields(query_params))

            query_params["$alt"] = "json;enum-encoding=int"

            # Send the request
            headers = dict(metadata)
            headers["Content-Type"] = "application/json"
            response = getattr(self._session, method)(
                "{host}{uri}".format(host=self._host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params, strict=True),
            )

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

            # Return the response
            resp = webrisk.SearchHashesResponse()
            pb_resp = webrisk.SearchHashesResponse.pb(resp)

            json_format.Parse(response.content, pb_resp, ignore_unknown_fields=True)
            resp = self._interceptor.post_search_hashes(resp)
            return resp

    class _SearchUris(WebRiskServiceV1Beta1RestStub):
        def __hash__(self):
            return hash("SearchUris")

        __REQUIRED_FIELDS_DEFAULT_VALUES: Dict[str, Any] = {
            "uri": "",
            "threatTypes": {},
        }

        @classmethod
        def _get_unset_required_fields(cls, message_dict):
            return {
                k: v
                for k, v in cls.__REQUIRED_FIELDS_DEFAULT_VALUES.items()
                if k not in message_dict
            }

        def __call__(
            self,
            request: webrisk.SearchUrisRequest,
            *,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Optional[float] = None,
            metadata: Sequence[Tuple[str, str]] = (),
        ) -> webrisk.SearchUrisResponse:
            r"""Call the search uris method over HTTP.

            Args:
                request (~.webrisk.SearchUrisRequest):
                    The request object. Request to check URI entries against
                threatLists.
                retry (google.api_core.retry.Retry): Designation of what errors, if any,
                    should be retried.
                timeout (float): The timeout for this request.
                metadata (Sequence[Tuple[str, str]]): Strings which should be
                    sent along with the request as metadata.

            Returns:
                ~.webrisk.SearchUrisResponse:

            """

            http_options: List[Dict[str, str]] = [
                {
                    "method": "get",
                    "uri": "/v1beta1/uris:search",
                },
            ]
            request, metadata = self._interceptor.pre_search_uris(request, metadata)
            pb_request = webrisk.SearchUrisRequest.pb(request)
            transcoded_request = path_template.transcode(http_options, pb_request)

            uri = transcoded_request["uri"]
            method = transcoded_request["method"]

            # Jsonify the query params
            query_params = json.loads(
                json_format.MessageToJson(
                    transcoded_request["query_params"],
                    including_default_value_fields=False,
                    use_integers_for_enums=True,
                )
            )
            query_params.update(self._get_unset_required_fields(query_params))

            query_params["$alt"] = "json;enum-encoding=int"

            # Send the request
            headers = dict(metadata)
            headers["Content-Type"] = "application/json"
            response = getattr(self._session, method)(
                "{host}{uri}".format(host=self._host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params, strict=True),
            )

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

            # Return the response
            resp = webrisk.SearchUrisResponse()
            pb_resp = webrisk.SearchUrisResponse.pb(resp)

            json_format.Parse(response.content, pb_resp, ignore_unknown_fields=True)
            resp = self._interceptor.post_search_uris(resp)
            return resp

    @property
    def compute_threat_list_diff(
        self,
    ) -> Callable[
        [webrisk.ComputeThreatListDiffRequest], webrisk.ComputeThreatListDiffResponse
    ]:
        # The return type is fine, but mypy isn't sophisticated enough to determine what's going on here.
        # In C++ this would require a dynamic_cast
        return self._ComputeThreatListDiff(self._session, self._host, self._interceptor)  # type: ignore

    @property
    def search_hashes(
        self,
    ) -> Callable[[webrisk.SearchHashesRequest], webrisk.SearchHashesResponse]:
        # The return type is fine, but mypy isn't sophisticated enough to determine what's going on here.
        # In C++ this would require a dynamic_cast
        return self._SearchHashes(self._session, self._host, self._interceptor)  # type: ignore

    @property
    def search_uris(
        self,
    ) -> Callable[[webrisk.SearchUrisRequest], webrisk.SearchUrisResponse]:
        # The return type is fine, but mypy isn't sophisticated enough to determine what's going on here.
        # In C++ this would require a dynamic_cast
        return self._SearchUris(self._session, self._host, self._interceptor)  # type: ignore

    @property
    def kind(self) -> str:
        return "rest"

    def close(self):
        self._session.close()


__all__ = ("WebRiskServiceV1Beta1RestTransport",)
